subroutine CalcEnergy(Pos, L, rc, PEnergy, BondPairs, LJPairs, Dim, NAtom, NBond,NLJ,LJPwidth)
    implicit none
    integer, intent(in) :: Dim, NAtom, NBond, NLJ, LJPwidth
    real(8), intent(in), dimension(0:NAtom-1, 0:Dim-1) :: Pos
    integer, intent(in), dimension(0:NBond-1) :: BondPairs
    integer, intent(in), dimension(0:NLJ-1, 0:LJPwidth-1) :: LJPairs
    real(8), intent(in) :: L, rc
    real(8), intent(out) :: PEnergy
    real(8), parameter :: k = 3000., r0 = 1.
    real(8), dimension(Dim) :: rij, Posi
    real(8) :: d2, id2, id6, id12
    real(8) :: rc2, ic2, ic6, Shift
    integer :: ind, i, j

    !compute constants outside pairwise loop
    !cutoff distance squared
    rc2 = rc*rc
    !shift factor
    ic2 = 1./rc2
    ic6 = ic2*ic2*ic2
    Shift = -4.*(ic6*ic6 - ic6)

    !reset PEnergy and Forces to 0 before calculation
    PEnergy = 0.
       
    !iterate i over all atoms
    do i = 0, NAtom -1

       !store Pos(i,:) for efficient access inside j loop
       Posi = Pos(i,:)

       !compute interaction for atom j>i bonded to i, if any
       !store value of j from list of bond pairs
       j = BondPairs(i)
       if (j /= -1) then

          !compute the squared distance between atoms i and j
          !shift as neccesary to account for periodic boundaries
          rij = Pos(j,:) - Posi
          !compute minimum squared image distance
          rij = rij - L*dnint(rij/L)
          d2 = sum(rij * rij)

          !use harmonic bond potential
          PEnergy = PEnergy + k/2.*(sqrt(d2)-r0)**2
       endif

       !iterate all pairs of bonded atoms
       do ind= 0, LJPwidth -1
          
          !store value of j from list of bond pairs
          j = LJPairs(i,ind)

          !if j is -1, there are no more atoms bonded to i
          if (j == -1) then
             exit
          endif

          !compute the squared distance between atoms i and j
          !shift as neccesary to account for periodic boundaries
          rij = Pos(j,:) - Posi
          !compute minimum squared image distance
          rij = rij - L*dnint(rij/L)
          d2 = sum(rij * rij)
          !advance to next atom if distance is above cutoff
          if (d2 > rc2) then
          cycle
       end if
       
       !use LJ potential with cut and sihft if non-bonded
       !compute inverse distance quantities, energy, and force
       id2 = 1./d2
       id6 = id2*id2*id2
       id12 = id6*id6
       PEnergy = PEnergy + 4.*(id12 - id6) + Shift
       
    enddo
 enddo
end subroutine


subroutine CalcEnergyForces(Pos, L, rc, PEnergy, Forces, BondPairs, LJPairs, Dim, NAtom, NBond,NLJ,LJPwidth)
    implicit none
    integer, intent(in) :: Dim, NAtom, NBond, NLJ, LJPwidth
    real(8), intent(in), dimension(0:NAtom-1, 0:Dim-1) :: Pos
    integer, intent(in), dimension(0:NBond-1) :: BondPairs
    integer, intent(in), dimension(0:NLJ-1, 0:LJPwidth-1) :: LJPairs
    real(8), intent(in) :: L, rc
    real(8), intent(out) :: PEnergy
    real(8), intent(inout), dimension(0:NAtom-1, 0:Dim-1) :: Forces
!f2py intent(in, out) :: Forces
    real(8), parameter :: k = 3000., r0 = 1.
    real(8), dimension(Dim) :: rij, Fij, Posi
    real(8) :: d, dr0, kdr0, d2, id2, id6, id12
    real(8) :: rc2, ic2, ic6, Shift
    integer :: ind, i, j

    !compute constants outside pairwise loop
    !cutoff distance squared
    rc2 = rc*rc
    !shift factor
    ic2 = 1./rc2
    ic6 = ic2*ic2*ic2
    Shift = -4.*(ic6*ic6 - ic6)

    !reset PEnergy and Forces to 0 before calculation
    PEnergy = 0.
    Forces = 0.
       
    !iterate i over all atoms
    do i = 0, NAtom -1

       !store Pos(i,:) for efficient access inside j loop
       Posi = Pos(i,:)

       !compute interaction for atom j>i bonded to i, if any
       !store value of j from list of bond pairs
       j = BondPairs(i)
       if (j /= -1) then

          !compute the squared distance between atoms i and j
          !shift as neccesary to account for periodic boundaries
          rij = Pos(j,:) - Posi
          !compute minimum squared image distance
          rij = rij - L*dnint(rij/L)
          d2 = sum(rij * rij)

          !use harmonic bond potential
          d = sqrt(d2)
          dr0 = d - r0
          kdr0 = k*dr0
          PEnergy = PEnergy + kdr0*dr0/2.
          Fij = rij/d*kdr0
          Forces(i,:) = Forces(i,:) + Fij
          Forces(j,:) = Forces(j,:) - Fij
       endif

       !iterate all pairs of bonded atoms
       do ind= 0, LJPwidth -1
          
          !store value of j from list of bond pairs
          j = LJPairs(i,ind)

          !if j is -1, there are no more atoms bonded to i
          if (j == -1) then
             exit
          endif

          !compute the squared distance between atoms i and j
          !shift as neccesary to account for periodic boundaries
          rij = Pos(j,:) - Posi
          !compute minimum squared image distance
          rij = rij - L*dnint(rij/L)
          d2 = sum(rij * rij)
          !advance to next atom if distance is above cutoff
          if (d2 > rc2) then
          cycle
       end if
       
       !use LJ potential with cut and sihft if non-bonded
       !compute inverse distance quantities, energy, and force
       id2 = 1./d2
       id6 = id2*id2*id2
       id12 = id6*id6
       PEnergy = PEnergy + 4.*(id12 - id6) + Shift
       Fij = rij*((-48.*id12 + 24.*id6)*id2)
       Forces(i,:) = Forces(i,:) + Fij
       Forces(j,:) = Forces(j,:) - Fij
       
    enddo
 enddo
end subroutine


subroutine VVIntegrate(Pos, Vel, Accel, L, rc, dt, KEnergy, PEnergy, BondPairs, LJPairs, Dim, NAtom, NBond,NLJ,LJPwidth)
    implicit none
    integer, intent(in) :: Dim, NAtom, NBond, NLJ, LJPwidth
    real(8), intent(in) :: L, rc, dt
    integer, intent(in), dimension(0:NBond-1) :: BondPairs
    integer, intent(in), dimension(0:NLJ-1, 0:LJPwidth-1) :: LJPairs
    real(8), intent(inout), dimension(0:NAtom-1, 0:Dim-1) :: Pos, Vel, Accel
!f2py intent(in,out) :: Pos, Vel, Accel
    real(8), intent(out) :: KEnergy, PEnergy
    external :: CalcEnergyForces
    Pos = Pos + dt * Vel + 0.5 * dt*dt * Accel
    Vel = Vel + 0.5 * dt * Accel
    call CalcEnergyForces(Pos, L, rc, PEnergy, Accel, BondPairs, LJPairs, Dim, NAtom, NBond,NLJ,LJPwidth)
    Vel = Vel + 0.5 * dt * Accel
    KEnergy = 0.5 * sum(Vel*Vel)
end subroutine

